@@ -10,23 +10,20 @@
 #include <ngx_event.h>
 
 
-#define NGX_IOVS  16
-
-
-#if (NGX_HAVE_KQUEUE)
-
 ssize_t
-ngx_readv_chain(ngx_connection_t *c, ngx_chain_t *chain)
+ngx_readv_chain(ngx_connection_t *c, ngx_chain_t *chain, off_t limit)
 {
     u_char        *prev;
     ssize_t        n, size;
     ngx_err_t      err;
     ngx_array_t    vec;
     ngx_event_t   *rev;
-    struct iovec  *iov, iovs[NGX_IOVS];
+    struct iovec  *iov, iovs[NGX_IOVS_PREALLOCATE];
 
     rev = c->read;
 
+#if (NGX_HAVE_KQUEUE)
+
     if (ngx_event_flags & NGX_USE_KQUEUE_EVENT) {
         ngx_log_debug3(NGX_LOG_DEBUG_EVENT, c->log, 0,
                        "readv: eof:%d, avail:%d, err:%d",
@@ -54,6 +51,8 @@ ngx_readv_chain(ngx_connection_t *c, ngx_chain_t *chain)
         }
     }
 
+#endif
+
     prev = NULL;
     iov = NULL;
     size = 0;
@@ -61,14 +60,26 @@ ngx_readv_chain(ngx_connection_t *c, ngx_chain_t *chain)
     vec.elts = iovs;
     vec.nelts = 0;
     vec.size = sizeof(struct iovec);
-    vec.nalloc = NGX_IOVS;
+    vec.nalloc = NGX_IOVS_PREALLOCATE;
     vec.pool = c->pool;
 
     /* coalesce the neighbouring bufs */
 
     while (chain) {
+        n = chain->buf->end - chain->buf->last;
+
+        if (limit) {
+            if (size >= limit) {
+                break;
+            }
+
+            if (size + n > limit) {
+                n = (ssize_t) (limit - size);
+            }
+        }
+
         if (prev == chain->buf->last) {
-            iov->iov_len += chain->buf->end - chain->buf->last;
+            iov->iov_len += n;
 
         } else {
             if (vec.nelts >= IOV_MAX) {
@@ -81,23 +92,24 @@ ngx_readv_chain(ngx_connection_t *c, ngx_chain_t *chain)
             }
 
             iov->iov_base = (void *) chain->buf->last;
-            iov->iov_len = chain->buf->end - chain->buf->last;
+            iov->iov_len = n;
         }
 
-        size += chain->buf->end - chain->buf->last;
+        size += n;
         prev = chain->buf->end;
         chain = chain->next;
     }
 
     ngx_log_debug2(NGX_LOG_DEBUG_EVENT, c->log, 0,
-                   "readv: %d, last:%d", vec.nelts, iov->iov_len);
-
-    rev = c->read;
+                   "readv: %ui, last:%uz", vec.nelts, iov->iov_len);
 
     do {
         n = readv(c->fd, (struct iovec *) vec.elts, vec.nelts);
 
         if (n >= 0) {
+
+#if (NGX_HAVE_KQUEUE)
+
             if (ngx_event_flags & NGX_USE_KQUEUE_EVENT) {
                 rev->available -= n;
 
@@ -137,6 +149,8 @@ ngx_readv_chain(ngx_connection_t *c, ngx_chain_t *chain)
                 return n;
             }
 
+#endif /* NGX_HAVE_KQUEUE */
+
             if (n < size && !(ngx_event_flags & NGX_USE_GREEDY_EVENT)) {
                 rev->ready = 0;
             }
@@ -170,98 +184,3 @@ ngx_readv_chain(ngx_connection_t *c, ngx_chain_t *chain)
 
     return n;
 }
-
-#else /* ! NGX_HAVE_KQUEUE */
-
-ssize_t
-ngx_readv_chain(ngx_connection_t *c, ngx_chain_t *chain)
-{
-    u_char        *prev;
-    ssize_t        n, size;
-    ngx_err_t      err;
-    ngx_array_t    vec;
-    ngx_event_t   *rev;
-    struct iovec  *iov, iovs[NGX_IOVS];
-
-    prev = NULL;
-    iov = NULL;
-    size = 0;
-
-    vec.elts = iovs;
-    vec.nelts = 0;
-    vec.size = sizeof(struct iovec);
-    vec.nalloc = NGX_IOVS;
-    vec.pool = c->pool;
-
-    /* coalesce the neighbouring bufs */
-
-    while (chain) {
-        if (prev == chain->buf->last) {
-            iov->iov_len += chain->buf->end - chain->buf->last;
-
-        } else {
-            if (vec.nelts >= IOV_MAX) {
-                break;
-            }
-
-            iov = ngx_array_push(&vec);
-            if (iov == NULL) {
-                return NGX_ERROR;
-            }
-
-            iov->iov_base = (void *) chain->buf->last;
-            iov->iov_len = chain->buf->end - chain->buf->last;
-        }
-
-        size += chain->buf->end - chain->buf->last;
-        prev = chain->buf->end;
-        chain = chain->next;
-    }
-
-    ngx_log_debug2(NGX_LOG_DEBUG_EVENT, c->log, 0,
-                   "readv: %d:%d", vec.nelts, iov->iov_len);
-
-    rev = c->read;
-
-    do {
-        n = readv(c->fd, (struct iovec *) vec.elts, vec.nelts);
-
-        if (n == 0) {
-            rev->ready = 0;
-            rev->eof = 1;
-
-            return n;
-
-        } else if (n > 0) {
-
-            if (n < size && !(ngx_event_flags & NGX_USE_GREEDY_EVENT)) {
-                rev->ready = 0;
-            }
-
-            return n;
-        }
-
-        err = ngx_socket_errno;
-
-        if (err == NGX_EAGAIN || err == NGX_EINTR) {
-            ngx_log_debug0(NGX_LOG_DEBUG_EVENT, c->log, err,
-                           "readv() not ready");
-            n = NGX_AGAIN;
-
-        } else {
-            n = ngx_connection_error(c, err, "readv() failed");
-            break;
-        }
-
-    } while (err == NGX_EINTR);
-
-    rev->ready = 0;
-
-    if (n == NGX_ERROR) {
-        c->read->error = 1;
-    }
-
-    return n;
-}
-
-#endif /* NGX_HAVE_KQUEUE */
